#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ˅
from configure_reader import *
from image_reader import *
import numpy as np

# ˄


class CalculateDisplayImage(object):
    def __init__(self):

        self.__ConfigureReader = None

        self.__ImageReader = None

        self.gamma = ConfigureReader.getGamma

        self.projector_image_width = ConfigureReader.getProjectorImageWidth
        
        self.np_left, self_np_right = ImageReader.getImage
        self.projector_distance = ConfigureReader.getProjectorDistance
    # calculate overlap pixels
        self.overlap_pixels = round(ConfigureReader.getImageWidth*(ConfigureReader.getProjectorImageWidth - ConfigureReader.getProjectorDistance)/ConfigureReader.getProjectorImageWidth)
        
    
    def calculate(self):
        # generate masks
        mask_left = self.maskGenerator(np.ones(self.np_left.shape), direction='left')
        mask_right = self.maskGenerator(np.ones(self.np_right.shape), direction='right')
        # blend images with masks
        with_mask_left = np.multiply(self.np_left, mask_left)
        with_mask_right = np.multiply(self.np_right, mask_right)
        # apply gamma correction
        blended_left = self.gammaCorrection(with_mask_left, direction='left', gamma=self.gamma)
        blended_right = self.gammaCorrection(with_mask_right, direction='right', gamma=self.gamma)
        return blended_left, blended_right

    def getDisplayImage(np_tuple1, np_tuple2):
        # to RGB
        blended_left = np.rint(np_tuple1 * 255).astype(np.uint8)
        blended_right = np.rint(np_tuple2 * 255).astype(np.uint8)
        # save images
        cv.imwrite('src/blended_left.jpg', blended_left)
        cv.imwrite('src/blended_right.jpg', blended_right)
        return blended_left, blended_right

    def gammaCorrection(self, direction, img):
        if direction == 'left':
            for rows in range(ConfigureReader.getImageHeight):
                for columns in range(ConfigureReader.getImageWidth - self.overlap_pixels-1, ConfigureReader.getImageWidth):
                    img[rows][columns][:] *= np.power(self.intensityController(
                    columns-(ConfigureReader.getImageWidth - self.overlap_pixels - 1), monotonicity='increase'), ConfigureReader.getGamma)
            return img

        elif direction == 'right':
            for rows in range(ConfigureReader.getImageHeight):
                for columns in range(self.overlap_pixels-1):
                    img[rows][columns][:] *= np.power(
                        self.intensityController(columns, monotonicity='decrease'), ConfigureReader.getGamma)
            return img
        
    def intensityController(self, distance, monotonicity):
        if monotonicity == 'increase':
            return -(1/self.overlap_pixels)*distance + 1
        elif monotonicity == 'decrease':
            return (1/self.overlap_pixels)*distance
        # a non-linear approach: -2*np.power(distance/overlap_pixels, 2)+1
    
    def getImageWidth(img):
        return img.shape[0]
    
    def getImageHeight(img):
        return img.shape[1]
    
    def maskGenerator(self, mask, direction):

        for columns in range(self.getImageWidth(mask)- self.overlap_pixels-1, self.getImageWidth(mask)):
            mask[0, columns, :] *= self.intensityController(
                columns-(self.getImageWidth(mask)- self.overlap_pixels-1), monotonicity='increase')
        # replacement is more efficient than multiplication
        for rows in range(1, self.getImageHeight(mask)):
            mask[rows, :, :] = mask[0, :, :]

        if direction == 'left':
            return mask
        elif direction == 'right':
            # flip direction of mask to right
            return np.flip(mask, axis=1)